function [p, resNorm, RESIDUAL, imageOfFit, weighted, paramNames] = myGauss2DFit_rotation(I,indices,initialGuess,lb, ub, W, maxPositionDisplacement, forcePeakAtMaxIntensity)

%I            - the image
%indices      - [ymin, ymax, xmin, xmax], the mesh will be generated by ymin:ymax and xmin:xmax
%initialGuess - [meanx meany sigmax sigmay cMult cAdd rho]
%the target function to be fitted is f(x,y) = cMult * Gauss(x,y;mx,my,sx,sy,rho) + cAdd;
%
%output p = [meanx meany sigmax sigmay cMult cAdd rho]
%       resNorm(1) ==> resNorm, resNorm(2) ==> gSNR
%see also: myGauss2DFit, myGauss2D_rotation2, myGauss2DFit_rotation2_mixture
%               1        2       3        4        5      6      7
paramNames = {'meanx' 'meany' 'sigmax' 'sigmay' 'cMult' 'cAdd' 'rho'};
objectiveFunction = @myGauss2D_rotation2;
global param;

    function D = diff_objectiveFunction_image(p, grid, I)
        D = objectiveFunction(p, grid) - I;
    end
    function D = diff_objectiveFunction_image_weighted(p, grid, I)
        D = diff_objectiveFunction_image(p, grid, I) .* W;
    end

if(nargin < 2 || isempty(indices))
    [X,Y] = meshgrid(1:size(I,2),1:size(I,1));
else
    [X,Y] = meshgrid(indices(3):indices(4),indices(1):indices(2));
end

grid = [X Y];
xmin = X(1);
ymin = Y(1);
xmax = X(end);
ymax = Y(end);

if(nargin >= 6 && ~isempty(W))
    functionToBeFitted = @diff_objectiveFunction_image_weighted;
    weighted = 1;
else
    functionToBeFitted = @diff_objectiveFunction_image;
    weighted = 0;
end

if(nargin < 7 || isempty(maxPositionDisplacement))
    try
        maxPositionDisplacement = param.ap.myGauss2DFit.maxPositionDisplacement;
    catch
        maxPositionDisplacement = [inf inf];
        printDebugStack(lasterror);
    end
end
if(length(maxPositionDisplacement) == 1)
    maxPositionDisplacement(2) = maxPositionDisplacement(1);
elseif(isempty(maxPositionDisplacement))
    maxPositionDisplacement = [inf inf];
end
if(length(maxPositionDisplacement) ~= 2)
    error('maxPositionDisplacement must have too elements at this point in the code');
end

if(nargin < 8)
    forcePeakAtMaxIntensity = 0;
end

I_smoothed = imsmooth(I,1);
maxI = max(I(:));
minI = min(I(:));
center = [X(1,ceil(end/2)),Y(ceil(end/2),1)];
if(nargin < 3 || isempty(initialGuess))
    r = min(maxPositionDisplacement, floor([size(I,2), size(I,1)]/2));
    [i,j,maxI_nearCenter] = index2DOfMax(I_smoothed,[ceil([size(I,2), size(I,1)]/2)-r, 2*r(2)+1, 2*r(1)+1]);
    x0_atMaxIntensity = j + X(1) - 1;
    y0_atMaxIntensity = i + Y(1) - 1;
    
    intensitiesSorted = sort(I_smoothed(:));
    bg_guess = mean(intensitiesSorted(1:ceil(end/2)));
    
    [alpha, majorLen, minorLen ] = estimateOrientationandSize(I_smoothed);
    majorLen = max(majorLen, 6);
    minorLen = max(minorLen, 6);
    
    %     initialGuess = [center(1),center(2),majorLen/4,minorLen/4, maxI_nearCenter-minI, minI, alpha];
    initialGuess = [x0_atMaxIntensity,y0_atMaxIntensity,majorLen/4,minorLen/4, maxI_nearCenter-bg_guess, bg_guess, alpha];
end



try
    minSigma                = param.ap.myGauss2DFit.minSigma / param.resolution;
    maxSigma                = param.ap.myGauss2DFit.maxSigma / param.resolution;
    
    if(nargin < 4 || strcmpi(lb,'auto'))
        xmin = max(xmin, center(1)-maxPositionDisplacement(1));
        ymin = max(ymin, center(2)-maxPositionDisplacement(2));
        %      mx   my      sx       sy      cMult   cAdd    rho
        lb = [xmin ymin   minSigma  minSigma   0      minI   -90];
    end
    
    if(nargin < 5 || strcmpi(ub,'auto'))
        xmax = min(xmax, center(1)+maxPositionDisplacement(1));
        ymax = min(ymax, center(2)+maxPositionDisplacement(2));
        %      mx   my      sx       sy      cMult   cAdd    rho
        ub = [xmax ymax  maxSigma  maxSigma  3*maxI  maxI   90];
    end
    if(lb(1) > initialGuess(1) || ub(1) < initialGuess(1))
        initialGuess(1) = center(1);
    end
    if(lb(2) > initialGuess(2) || ub(2) < initialGuess(2))
        initialGuess(2) = center(2);
    end
catch
    printDebugStack(lasterror);
    lb = [];
    ub = [];
end

curvefitoptions = optimset('Display','off');
% curvefitoptions = optimset('Display','off','TolFun',1e-10, 'MaxIter', 10000);
[p,resNorm,RESIDUAL] = lsqnonlin(functionToBeFitted,initialGuess,lb,ub,curvefitoptions,grid,I);

if(forcePeakAtMaxIntensity)
    % force peak to be at max intensity level
    radiusMaxRegion = ceil(p(3)+p(4));
    [i,j,maxI_nearCenter] = index2DOfMax(I,[ceil([size(I,2), size(I,1)]/2)-radiusMaxRegion, 2*radiusMaxRegion+1, 2*radiusMaxRegion+1]);
    lb(5) =  .99*(maxI_nearCenter-p(6));
    p(5)  = maxI_nearCenter-p(6);
    [p,resNorm,RESIDUAL] = lsqnonlin(functionToBeFitted,p,lb,ub,curvefitoptions,grid,I);
end

if(p(3) < p(4))
    %     warning('sigma_major < simga_minor');
    sigmax = p(3);
    p(3) = p(4);
    p(4) = sigmax;
    if(p(7) >= 0)
        p(7) = 90-p(7);
    else
        p(7) = p(7)+90;
    end
end

try
    % compute SNR
    imageOfFit = objectiveFunction( p, grid );
    bg = p(6);
    I_minusBG = I - bg;
    p_Norm = p;
    p_Norm(1:2) = 0;
    p_Norm(7)   = 0;
    x0 = 2*p(3);
    y0 = 2*p(4);
    intensityAt2Sigma = objectiveFunction(p_Norm, [x0 y0]);
    maskSignal = imageOfFit >= intensityAt2Sigma;
    maskBackground = ~maskSignal;
    [SNR, r, signalMean, noise, signal, bg] = ComputeSNR(I_minusBG, maskSignal, maskBackground);
    if(r == inf)
        r = -1;
    end
catch
    r = -1;
end
resNorm(2) = r;


if(getDebugLevel()>=2)
    figure;
    imageOfFit_initialGuess     = objectiveFunction( initialGuess, grid );
    meanResNorm_initialGuess    = (imageOfFit_initialGuess - I).^2;
    meanResNorm_initialGuess    = mean(meanResNorm_initialGuess(:)) * 1000000;
    imageOfFit                  = objectiveFunction( p, grid );
    meanResNorm                 = (imageOfFit - I).^2;
    meanResNorm                 = mean(meanResNorm(:)) * 1000000;
    subplot(1,2,1)
    txt_title = sprintf('initial guess\n%s',printFittingParameters([paramNames {'mean resNorm'}],[initialGuess meanResNorm_initialGuess]));
    displayFit(I, imageOfFit_initialGuess, txt_title);  axis square
    subplot(1,2,2)
    txt_title = sprintf('final\n%s',printFittingParameters([paramNames {'mean resNorm'}],[p meanResNorm]));
    displayFit(I, imageOfFit, txt_title); axis square
end

end

function [alpha, majorLen, minorLen ] = estimateOrientationandSize(I)
% Ithr = otsu(I);
maxIntensitySearchRange = [3 3];
maxCorrCoef = -.7;
imageCroppingStepsize = 0;
minImageSize = size(I);
maxWeightLeftRight = 6/7;
[thrLevel, cutpointRadius, maxIntensityPoint, thrLevelRange, Ithr ] = SpotThresholding( I, maxIntensitySearchRange, maxCorrCoef, imageCroppingStepsize, minImageSize, maxWeightLeftRight );


rp = myregionprops(bwlabel(Ithr), 'Area', 'Orientation', 'MajorAxisLength','MinorAxisLength', 'Centroid', 'ShapeFactor');
Nregions = length(rp);
if(isempty(rp))
    sf = 0;
    r = 0;
    if(getDebugLevel()>=2)
        figure;
        subplot(2,2,1), imshow(I,[]); title('(binary image is empty)');
    end
else
    [max_area, max_area_idx] = max([rp.Area]);
    
    sf = rp(max_area_idx).ShapeFactor;
    p = rp(max_area_idx).Centroid;
    Dist2Intensity = radialIntensityProfile( Ithr, p([2 1]), 'integerDistances');
    % Dist2Intensity = radialIntensityProfile( I, ceil(size(I)/2), 'integerDistances','normalize');
    
    d = diff(Dist2Intensity(:,2));
    sum_p = sum(d(d>0));
    sum_n = sum(abs(d(d<0)));
    r = sum_n / sum_p; %the greater r the more is the curve negative
    if(r > 100)
        r = 100;
    end
    if(getDebugLevel()>=2)
        corrcoef_ = corrcoef(Dist2Intensity(:,1),Dist2Intensity(:,2));
        corrcoef_ = corrcoef_(end,1); %usually corrcoef_(end,1) means corrcoef_(2,1), but if there is only one value it indexes to corrcoef_(1,1)!
        figure;
        subplot(2,2,1), imshow(I,[]);
        subplot(2,2,3), imshow(Ithr,[]); title(sprintf('|regions| = %d, shape factor = %.2f',Nregions,sf));
        subplot(2,2,2);
        plot(Dist2Intensity(:,1),Dist2Intensity(:,2), 'o-'); axis square
        title(sprintf('corrcoef = %f, sum_p = %f, sum_n = %f, sum_n / sum_p = %f',corrcoef_, sum_p, sum_n, r))
    end
    
end


if(r > 5 && sf > .6)
    alpha = rp(max_area_idx).Orientation;
    majorLen = rp(max_area_idx).MajorAxisLength;
    minorLen = rp(max_area_idx).MinorAxisLength;
else %
    alpha = 0;
    majorLen = 7;
    minorLen = 7;
end
end


